// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_POSIX_CLIENTSOCKET_H
#define HU_VARADIISTVAN_SCPL_IO_POSIX_CLIENTSOCKET_H
//------------------------------------------------------------------------------

#include "Socket.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Connector;

//------------------------------------------------------------------------------

/**
 * A socket that can be connected to another one.
 */
class ClientSocket : public Socket
{
private:
    /**
     * The connector.
     */
    Connector& connector;

protected:
    /**
     * Construct the socket with the given parameters.
     */
    ClientSocket(int domain, int type, Connector& connector,
                 Waiter* waiter = 0, int protocol = 0,
                 size_t readingCapacity = DEFAULT_CAPACITY,
                 size_t writingCapacity = DEFAULT_CAPACITY);

protected:
    /**
     * Handle any events on the file descriptor. If a connection is in
     * progress, and the incoming events have the EPOLLOUT flag set,
     * the connector is called to handle the event. Then the parent
     * class' handleEvents function is called.
     */
    virtual void handleEvents(short events);

    /**
     * Connect the socket to the given address.
     *
     * @return whether the connection succeeded or not. If it does not
     * succeeded, it may be either because of the socket being
     * non-blocking and some time is needed, or because of a failure,
     * in which case the error code will be set properly.
     */
    bool connect(const struct sockaddr* addr, size_t addrlen);

    friend class Connector;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline ClientSocket::ClientSocket(int domain, int type, Connector& connector,
                                  Waiter* waiter, int protocol,
                                  size_t readingCapacity,
                                  size_t writingCapacity) :
    Socket(domain, type, waiter, protocol, readingCapacity, writingCapacity),
    connector(connector)
{
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_POSIX_CLIENTSOCKET_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
